@@ -1,29 +1,17 @@
 """
 Protein-Ligand Interaction Profiler - Analyze and visualize protein-ligand interactions in PDB files.
 preparation.py - Prepare PDB input files for processing.
-Copyright 2014-2015 Sebastian Salentin
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
 """
 
 
 # Python Standard Library
+from __future__ import absolute_import
 from operator import itemgetter
 
 # Own modules
-from detection import *
-from supplemental import *
-import config
+from plip.modules.detection import *
+from plip.modules.supplemental import *
+import plip.modules.config
 
 
 ################
@@ -38,6 +26,7 @@ class PDBParser:
         self.covlinkage = namedtuple("covlinkage", "id1 chain1 pos1 conf1 id2 chain2 pos2 conf2")
         self.proteinmap, self.modres, self.covalent, self.altconformations, self.corrected_pdb = self.parse_pdb()
 
+
     def parse_pdb(self):
         """Extracts additional information from PDB files.
         I. When reading in a PDB file, OpenBabel numbers ATOMS and HETATOMS continously.
@@ -64,11 +53,20 @@ class PDBParser:
         if not config.NOFIX:
             if not config.PLUGIN_MODE:
                 lastnum = 0 # Atom numbering (has to be consecutive)
+                other_models = False
                 for line in fil:
-                    corrected_line, newnum = self.fix_pdbline(line, lastnum)
-                    if corrected_line is not None:
-                        corrected_lines.append(corrected_line)
-                        lastnum = newnum
+                    if not other_models: # Only consider the first model in an NRM structure
+                        corrected_line, newnum = self.fix_pdbline(line, lastnum)
+                        if corrected_line is not None:
+                            if corrected_line.startswith('MODEL'):
+                                try: # Get number of MODEL (1,2,3)
+                                    model_num = int(corrected_line[10:14])
+                                    if model_num > 1: # MODEL 2,3,4 etc.
+                                        other_models = True
+                                except ValueError:
+                                    write_message("Ignoring invalid MODEL entry: %s\n" % corrected_line, mtype='debug')
+                            corrected_lines.append(corrected_line)
+                            lastnum = newnum
                 corrected_pdb = ''.join(corrected_lines)
             else:
                 corrected_pdb = self.pdbpath
@@ -156,12 +154,16 @@ class PDBParser:
                     self.num_fixed_lines += 1
         if pdbline.startswith('HETATM'):
             newnum = lastnum + 1
-            currentnum = int(pdbline[6:11])
+            try:
+                currentnum = int(pdbline[6:11])
+            except ValueError:
+                currentnum = None
+                write_message("Invalid HETATM entry: %s\n" % pdbline, mtype='debug')
             if lastnum + 1 != currentnum:
                 pdbline = pdbline[:6] + (5 - len(str(newnum))) * ' ' + str(newnum) + ' ' + pdbline[12:]
                 fixed = True
             # No chain assigned or number assigned as chain
-            if pdbline[21] == ' ' or re.match("[0-9]", pdbline[21]):
+            if pdbline[21] == ' ':
                 pdbline = pdbline[:21] + 'Z' + pdbline[22:]
                 fixed = True
             # No residue number assigned
@@ -221,7 +223,6 @@ class LigandFinder:
         if len(all_from_chain) == 0:
             return None
         else:
-            #TODO Can be done shorter with a split
             water = [o for o in all_from_chain if o.GetResidueProperty(9)]
             non_water = water = [o for o in all_from_chain if not o.GetResidueProperty(9)]
             ligand = self.extract_ligand(non_water)
@@ -250,8 +251,12 @@ class LigandFinder:
                 res_kmers = self.identify_kmers(all_res_dict)
             else:
                 res_kmers = [[a, ] for a in ligand_residues]
+            write_message("{} ligand kmer(s) detected for closer inspection.\n".format(len(res_kmers)), mtype='debug')
             for kmer in res_kmers:  # iterate over all ligands and extract molecules + information
-                ligands.append(self.extract_ligand(kmer))
+                if len(kmer) > config.MAX_COMPOSITE_LENGTH:
+                    write_message("Ligand kmer(s) filtered out with a length of {} fragments ({} allowed).\n".format(len(kmer), config.MAX_COMPOSITE_LENGTH), mtype='debug')
+                else:
+                    ligands.append(self.extract_ligand(kmer))
 
         else:
             # Extract peptides from given chains
@@ -272,7 +277,7 @@ class LigandFinder:
         members = [(res.GetName(), res.GetChain(), int32_to_negative(res.GetNum())) for res in kmer]
         members = sort_members_by_importance(members)
         rname, rchain, rnum = members[0]
-        write_message("Finalizing extraction for ligand %s:%s:%s\n" % (rname, rchain, rnum), mtype='debug')
+        write_message("Finalizing extraction for ligand %s:%s:%s with %i elements\n" % (rname, rchain, rnum, len(kmer)), mtype='debug')
         names = [x[0] for x in members]
         longname = '-'.join([x[0] for x in members])
 
@@ -283,6 +288,7 @@ class LigandFinder:
         else:
             # Classify a ligand by its HETID(s)
             ligtype = classify_by_name(names)
+        write_message("Ligand classified as {}\n".format(ligtype), mtype='debug')
 
         hetatoms = set()
         for obresidue in kmer:
@@ -295,6 +301,7 @@ class LigandFinder:
                                     if not self.mapper.mapid(atm[0], mtype='protein',
                                                              to='internal') in self.altconformations])
             hetatoms.update(hetatoms_res)
+        write_message("Hetero atoms determined (n={})\n".format(len(hetatoms)), mtype='debug')
 
         hetatoms = dict(hetatoms)  # make it a dict with idx as key and OBAtom as value
         lig = pybel.ob.OBMol()  # new ligand mol
@@ -305,6 +312,7 @@ class LigandFinder:
             # ids of all neighbours of obatom
             neighbours[idx] = set([neighbour_atom.GetIdx() for neighbour_atom
                                    in pybel.ob.OBAtomAtomIter(obatom)]) & set(hetatoms.keys())
+        write_message("Atom neighbours mapped\n", mtype='debug')
 
         ##############################################################
         # map the old atom idx of OBMol to the new idx of the ligand #
@@ -329,6 +337,8 @@ class LigandFinder:
         lig.title = ':'.join((rname, rchain, str(rnum)))
         self.mapper.ligandmaps[lig.title] = mapold
 
+        write_message("Renumerated molecule generated\n", mtype='debug')
+
         atomorder = canonicalize(lig)
 
         can_to_pdb = {}
@@ -354,7 +364,7 @@ class LigandFinder:
             # Only residues with at least one HETATM entry are processed as ligands
             het_atoms = []
             for atm in pybel.ob.OBResidueAtomIter(obres):
-                    het_atoms.append(obres.IsHetAtom(atm))
+                het_atoms.append(obres.IsHetAtom(atm))
             if True in het_atoms:
                 return True
         return False
@@ -364,6 +374,9 @@ class LigandFinder:
         """Given an OpenBabel Molecule, get all ligands, their names, and water"""
 
         candidates1 = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if not o.GetResidueProperty(9) and self.is_het_residue(o)]
+
+        if config.DNARECEPTOR: # If DNA is the receptor, don't consider DNA as a ligand
+            candidates1 = [res for res in candidates1 if res.GetName() not in config.DNA+config.RNA]
         all_lignames = set([a.GetName() for a in candidates1])
 
         water = [o for o in pybel.ob.OBResidueIter(self.proteincomplex.OBMol) if o.GetResidueProperty(9)]
@@ -396,7 +409,8 @@ class LigandFinder:
                        (link.id2, link.chain2, link.pos2)] for link in
                       [c for c in self.covalent if c.id1 in self.lignames_kept and c.id2 in self.lignames_kept and
                        c.conf1 in ['A', ''] and c.conf2 in ['A', '']
-                      and (c.id1, c.chain1, c.pos1) in residues and (c.id2, c.chain2, c.pos2) in residues]]
+                      and (c.id1, c.chain1, c.pos1) in residues
+                      and (c.id2, c.chain2, c.pos2) in residues]]
         kmers = cluster_doubles(ligdoubles)
         if not kmers:  # No ligand kmers, just normal independent ligands
             return [[residues[res]] for res in residues]
@@ -422,10 +436,11 @@ class Mapper:
     def __init__(self):
         self.proteinmap = None  # Map internal atom IDs of protein residues to original PDB Atom IDs
         self.ligandmaps = {}  # Map IDs of new ligand molecules to internal IDs (or PDB IDs?)
+        self.original_structure = None
 
     def mapid(self, idx, mtype, bsid=None, to='original'):  # Mapping to original IDs is standard for ligands
-        # #@todo Check for correct type of idx
-        # #@todo Include more safety checks
+        if mtype == 'reversed': # Needed to map internal ID back to original protein ID
+            return self.reversed_proteinmap[idx]
         if mtype == 'protein':
             return self.proteinmap[idx]
         elif mtype == 'ligand':
@@ -434,7 +449,13 @@ class Mapper:
             elif to == 'original':
                 return self.proteinmap[self.ligandmaps[bsid][idx]]
 
-    # #@todo Include chains, resnr and positions here!
+    def id_to_atom(self, idx):
+        """Returns the atom for a given original ligand ID.
+        To do this, the ID is mapped to the protein first and then the atom returned.
+        """
+        mapped_idx = self.mapid(idx, 'reversed')
+        return pybel.Atom(self.original_structure.GetAtom(mapped_idx))
+
 
 
 class Mol:
@@ -452,47 +473,51 @@ class Mol:
     def hydrophobic_atoms(self, all_atoms):
         """Select all carbon atoms which have only carbons and/or hydrogens as direct neighbors."""
         atom_set = []
-        data = namedtuple('hydrophobic', 'atom orig_idx')
+        data = namedtuple('hydrophobic', 'atom orig_atom orig_idx')
         atm = [a for a in all_atoms if a.atomicnum == 6 and set([natom.GetAtomicNum() for natom
                                                                  in pybel.ob.OBAtomAtomIter(a.OBAtom)]).issubset(
             {1, 6})]
         for atom in atm:
             orig_idx = self.Mapper.mapid(atom.idx, mtype=self.mtype, bsid=self.bsid)
+            orig_atom = self.Mapper.id_to_atom(orig_idx)
             if atom.idx not in self.altconf:
-                atom_set.append(data(atom=atom, orig_idx=orig_idx))
+                atom_set.append(data(atom=atom, orig_atom=orig_atom, orig_idx=orig_idx))
         return atom_set
 
     def find_hba(self, all_atoms):
         """Find all possible hydrogen bond acceptors"""
-        data = namedtuple('hbondacceptor', 'a a_orig_idx type')
+        data = namedtuple('hbondacceptor', 'a a_orig_atom a_orig_idx type')
         a_set = []
         for atom in itertools.ifilter(lambda at: at.OBAtom.IsHbondAcceptor(), all_atoms):
             if atom.atomicnum not in [9, 17, 35, 53] and atom.idx not in self.altconf:  # Exclude halogen atoms
                 a_orig_idx = self.Mapper.mapid(atom.idx, mtype=self.mtype, bsid=self.bsid)
-                a_set.append(data(a=atom, a_orig_idx=a_orig_idx, type='regular'))
+                a_orig_atom = self.Mapper.id_to_atom(a_orig_idx)
+                a_set.append(data(a=atom, a_orig_atom=a_orig_atom, a_orig_idx=a_orig_idx, type='regular'))
         return a_set
 
     def find_hbd(self, all_atoms, hydroph_atoms):
         """Find all possible strong and weak hydrogen bonds donors (all hydrophobic C-H pairings)"""
         donor_pairs = []
-        data = namedtuple('hbonddonor', 'd d_orig_idx h type')
+        data = namedtuple('hbonddonor', 'd d_orig_atom d_orig_idx h type')
         for donor in [a for a in all_atoms if a.OBAtom.IsHbondDonor() and a.idx not in self.altconf]:
             in_ring = False
             if not in_ring:
                 for adj_atom in [a for a in pybel.ob.OBAtomAtomIter(donor.OBAtom) if a.IsHbondDonorH()]:
                     d_orig_idx = self.Mapper.mapid(donor.idx, mtype=self.mtype, bsid=self.bsid)
-                    donor_pairs.append(data(d=donor, d_orig_idx=d_orig_idx, h=pybel.Atom(adj_atom), type='regular'))
+                    d_orig_atom = self.Mapper.id_to_atom(d_orig_idx)
+                    donor_pairs.append(data(d=donor, d_orig_atom=d_orig_atom, d_orig_idx=d_orig_idx, h=pybel.Atom(adj_atom), type='regular'))
         for carbon in hydroph_atoms:
             for adj_atom in [a for a in pybel.ob.OBAtomAtomIter(carbon.atom.OBAtom) if a.GetAtomicNum() == 1]:
                 d_orig_idx = self.Mapper.mapid(carbon.atom.idx, mtype=self.mtype, bsid=self.bsid)
-                donor_pairs.append(data(d=carbon, d_orig_idx=d_orig_idx, h=pybel.Atom(adj_atom), type='weak'))
+                d_orig_atom = self.Mapper.id_to_atom(d_orig_idx)
+                donor_pairs.append(data(d=carbon, d_orig_atom=d_orig_atom, d_orig_idx=d_orig_idx, h=pybel.Atom(adj_atom), type='weak'))
         return donor_pairs
 
 
     def find_rings(self, mol, all_atoms):
         """Find rings and return only aromatic.
         Rings have to be sufficiently planar OR be detected by OpenBabel as aromatic."""
-        data = namedtuple('aromatic_ring', 'atoms atoms_orig_idx normal obj center type')
+        data = namedtuple('aromatic_ring', 'atoms orig_atoms atoms_orig_idx normal obj center type')
         rings = []
         aromatic_amino = ['TYR', 'TRP', 'HIS', 'PHE']
         ring_candidates = mol.OBMol.GetSSSR()
@@ -511,7 +536,9 @@ class Mol:
                     ringv1 = vector(ring_atms[0], ring_atms[1])
                     ringv2 = vector(ring_atms[2], ring_atms[0])
                     atoms_orig_idx = [self.Mapper.mapid(r_atom.idx, mtype=self.mtype, bsid=self.bsid) for r_atom in r_atoms]
+                    orig_atoms = [self.Mapper.id_to_atom(idx) for idx in atoms_orig_idx]
                     rings.append(data(atoms=r_atoms,
+                                  orig_atoms=orig_atoms,
                                   atoms_orig_idx=atoms_orig_idx,
                                   normal=normalize_vector(np.cross(ringv1, ringv2)),
                                   obj=ring,
@@ -981,13 +1008,14 @@ class Ligand(Mol):
 
         ######
         donor_pairs = []
-        data = namedtuple('hbonddonor', 'd d_orig_idx h type')
+        data = namedtuple('hbonddonor', 'd d_orig_atom d_orig_idx h type')
         for donor in self.all_atoms:
             pdbidx = self.Mapper.mapid(donor.idx, mtype='ligand', bsid=self.bsid, to='original')
             d = cclass.atoms[self.pdb_to_idx_mapping[pdbidx]]
             if d.OBAtom.IsHbondDonor():
                 for adj_atom in [a for a in pybel.ob.OBAtomAtomIter(d.OBAtom) if a.IsHbondDonorH()]:
-                    donor_pairs.append(data(d=donor, d_orig_idx=pdbidx, h=pybel.Atom(adj_atom), type='regular'))
+                    d_orig_atom = self.Mapper.id_to_atom(pdbidx)
+                    donor_pairs.append(data(d=donor, d_orig_atom=d_orig_atom, d_orig_idx=pdbidx, h=pybel.Atom(adj_atom), type='regular'))
         self.hbond_don_atom_pairs = donor_pairs
         #######
 
@@ -1009,9 +1037,11 @@ class Ligand(Mol):
         self.halogenbond_don = self.find_hal(self.all_atoms)
         self.metal_binding = self.find_metal_binding(self.all_atoms, self.water)
         self.metals = []
-        data = namedtuple('metal', 'm m_orig_idx')
+        data = namedtuple('metal', 'm orig_m m_orig_idx')
         for a in [a for a in self.all_atoms if a.type.upper() in config.METAL_IONS]:
-            self.metals.append(data(m=a, m_orig_idx=self.Mapper.mapid(a.idx, mtype=self.mtype, bsid=self.bsid)))
+            m_orig_idx = self.Mapper.mapid(a.idx, mtype=self.mtype, bsid=self.bsid)
+            orig_m = self.Mapper.id_to_atom(m_orig_idx)
+            self.metals.append(data(m=a, m_orig_idx=m_orig_idx, orig_m=orig_m))
         self.num_hba, self.num_hbd = len(self.hbond_acc_atoms), len(self.hbond_don_atom_pairs)
         self.num_hal = len(self.halogenbond_don)
 
@@ -1064,14 +1094,15 @@ class Ligand(Mol):
 
     def find_hal(self, atoms):
         """Look for halogen bond donors (X-C, with X=F, Cl, Br, I)"""
-        data = namedtuple('hal_donor', 'x x_orig_idx c c_orig_idx')
+        data = namedtuple('hal_donor', 'x orig_x x_orig_idx c c_orig_idx')
         a_set = []
         for a in atoms:
             if self.is_functional_group(a, 'halocarbon'):
                 n_atoms = [na for na in pybel.ob.OBAtomAtomIter(a.OBAtom) if na.GetAtomicNum() == 6]
                 x_orig_idx = self.Mapper.mapid(a.idx, mtype=self.mtype, bsid=self.bsid)
+                orig_x = self.Mapper.id_to_atom(x_orig_idx)
                 c_orig_idx = [self.Mapper.mapid(na.GetIdx(), mtype=self.mtype, bsid=self.bsid) for na in n_atoms]
-                a_set.append(data(x=a, x_orig_idx=x_orig_idx, c=pybel.Atom(n_atoms[0]), c_orig_idx=c_orig_idx))
+                a_set.append(data(x=a, orig_x=orig_x, x_orig_idx=x_orig_idx, c=pybel.Atom(n_atoms[0]), c_orig_idx=c_orig_idx))
         if len(a_set) != 0:
             write_message('Ligand contains %i halogen atom(s).\n' % len(a_set), indent=True)
         return a_set
@@ -1082,18 +1113,19 @@ class Ligand(Mol):
         as mentioned in 'Cation-pi interactions in ligand recognition and catalysis' (Zacharias et al., 2002)).
         Identify negatively charged groups in the ligand.
         """
-        data = namedtuple('lcharge', 'atoms atoms_orig_idx type center fgroup')
+        data = namedtuple('lcharge', 'atoms orig_atoms atoms_orig_idx type center fgroup')
         a_set = []
         for a in all_atoms:
             a_orig_idx = self.Mapper.mapid(a.idx, mtype=self.mtype, bsid=self.bsid)
+            a_orig = self.Mapper.id_to_atom(a_orig_idx)
             if self.is_functional_group(a, 'quartamine'):
-                a_set.append(data(atoms=[a, ], atoms_orig_idx=[a_orig_idx, ], type='positive',
+                a_set.append(data(atoms=[a, ], orig_atoms = [a_orig, ], atoms_orig_idx=[a_orig_idx, ], type='positive',
                                   center=list(a.coords), fgroup='quartamine'))
             elif self.is_functional_group(a, 'tertamine'):
-                a_set.append(data(atoms=[a, ], atoms_orig_idx=[a_orig_idx, ], type='positive', center=list(a.coords),
+                a_set.append(data(atoms=[a, ], orig_atoms = [a_orig, ], atoms_orig_idx=[a_orig_idx, ], type='positive', center=list(a.coords),
                                   fgroup='tertamine'))
             if self.is_functional_group(a, 'sulfonium'):
-                a_set.append(data(atoms=[a, ], atoms_orig_idx=[a_orig_idx, ], type='positive', center=list(a.coords),
+                a_set.append(data(atoms=[a, ], orig_atoms = [a_orig, ], atoms_orig_idx=[a_orig_idx, ], type='positive', center=list(a.coords),
                                   fgroup='sulfonium'))
             if self.is_functional_group(a, 'phosphate'):
                 a_contributing = [a, ]
@@ -1101,7 +1133,8 @@ class Ligand(Mol):
                 [a_contributing.append(pybel.Atom(neighbor)) for neighbor in pybel.ob.OBAtomAtomIter(a.OBAtom)]
                 [a_contributing_orig_idx.append(self.Mapper.mapid(neighbor.idx, mtype=self.mtype, bsid=self.bsid))
                  for neighbor in a_contributing]
-                a_set.append(data(atoms=a_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
+                orig_contributing = [self.Mapper.id_to_atom(idx) for idx in a_contributing_orig_idx]
+                a_set.append(data(atoms=a_contributing, orig_atoms=orig_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
                                   center=a.coords, fgroup='phosphate'))
             if self.is_functional_group(a, 'sulfonicacid'):
                 a_contributing = [a, ]
@@ -1110,7 +1143,8 @@ class Ligand(Mol):
                  neighbor.GetAtomicNum() == 8]
                 [a_contributing_orig_idx.append(self.Mapper.mapid(neighbor.idx, mtype=self.mtype, bsid=self.bsid))
                  for neighbor in a_contributing]
-                a_set.append(data(atoms=a_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
+                orig_contributing = [self.Mapper.id_to_atom(idx) for idx in a_contributing_orig_idx]
+                a_set.append(data(atoms=a_contributing, orig_atoms=orig_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
                                   center=a.coords, fgroup='sulfonicacid'))
             elif self.is_functional_group(a, 'sulfate'):
                 a_contributing = [a, ]
@@ -1118,21 +1152,24 @@ class Ligand(Mol):
                 [a_contributing_orig_idx.append(self.Mapper.mapid(neighbor.idx, mtype=self.mtype, bsid=self.bsid))
                  for neighbor in a_contributing]
                 [a_contributing.append(pybel.Atom(neighbor)) for neighbor in pybel.ob.OBAtomAtomIter(a.OBAtom)]
-                a_set.append(data(atoms=a_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
+                orig_contributing = [self.Mapper.id_to_atom(idx) for idx in a_contributing_orig_idx]
+                a_set.append(data(atoms=a_contributing, orig_atoms=orig_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
                                   center=a.coords, fgroup='sulfate'))
             if self.is_functional_group(a, 'carboxylate'):
                 a_contributing = [pybel.Atom(neighbor) for neighbor in pybel.ob.OBAtomAtomIter(a.OBAtom)
                                   if neighbor.GetAtomicNum() == 8]
                 a_contributing_orig_idx = [self.Mapper.mapid(neighbor.idx, mtype=self.mtype, bsid=self.bsid)
                                            for neighbor in a_contributing]
-                a_set.append(data(atoms=a_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
+                orig_contributing = [self.Mapper.id_to_atom(idx) for idx in a_contributing_orig_idx]
+                a_set.append(data(atoms=a_contributing, orig_atoms=orig_contributing, atoms_orig_idx=a_contributing_orig_idx, type='negative',
                                   center=centroid([a.coords for a in a_contributing]), fgroup='carboxylate'))
             elif self.is_functional_group(a, 'guanidine'):
                 a_contributing = [pybel.Atom(neighbor) for neighbor in pybel.ob.OBAtomAtomIter(a.OBAtom)
                                   if neighbor.GetAtomicNum() == 7]
                 a_contributing_orig_idx = [self.Mapper.mapid(neighbor.idx, mtype=self.mtype, bsid=self.bsid)
                                            for neighbor in a_contributing]
-                a_set.append(data(atoms=a_contributing, atoms_orig_idx=a_contributing_orig_idx, type='positive',
+                orig_contributing = [self.Mapper.id_to_atom(idx) for idx in a_contributing_orig_idx]
+                a_set.append(data(atoms=a_contributing, orig_atoms=orig_contributing, atoms_orig_idx=a_contributing_orig_idx, type='positive',
                                   center=a.coords, fgroup='guanidine'))
         return a_set
 
@@ -1142,11 +1179,11 @@ class Ligand(Mol):
         nitrogen from imidazole; sulfur from thiolate.
         """
         a_set = []
-        data = namedtuple('metal_binding', 'atom atom_orig_idx type fgroup restype resnr reschain location')
+        data = namedtuple('metal_binding', 'atom orig_atom atom_orig_idx type fgroup restype resnr reschain location')
         for oxygen in water_oxygens:
             a_set.append(data(atom=oxygen.oxy, atom_orig_idx=oxygen.oxy_orig_idx, type='O', fgroup='water',
                               restype=whichrestype(oxygen.oxy), resnr=whichresnumber(oxygen.oxy),
-                              reschain=whichchain(oxygen.oxy), location='water'))
+                              reschain=whichchain(oxygen.oxy), location='water', orig_atom=self.Mapper.id_to_atom(oxygen.oxy_orig_idx)))
         # #@todo Refactor code
         for a in lig_atoms:
             a_orig_idx = self.Mapper.mapid(a.idx, mtype='ligand', bsid=self.bsid)
@@ -1157,11 +1194,11 @@ class Ligand(Mol):
                 if n_atoms_atomicnum.count('1') == 1 and len(n_atoms_atomicnum) == 2:  # Oxygen in alcohol (R-[O]-H)
                     a_set.append(data(atom=a, atom_orig_idx=a_orig_idx, type='O', fgroup='alcohol',
                                       restype=self.hetid, resnr=self.position, reschain=self.chain,
-                                      location='ligand'))
+                                      location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
                 if True in [n.IsAromatic() for n in n_atoms] and not a.OBAtom.IsAromatic():  # Phenolate oxygen
                     a_set.append(data(atom=a, atom_orig_idx=a_orig_idx, type='O', fgroup='phenolate',
                                       restype=self.hetid, resnr=self.position, reschain=self.chain,
-                                      location='ligand'))
+                                      location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
             if a.atomicnum == 6:  # It's a carbon atom
                 if n_atoms_atomicnum.count(8) == 2 and n_atoms_atomicnum.count(6) == 1:  # It's a carboxylate group
                     for neighbor in [n for n in n_atoms if n.GetAtomicNum() == 8]:
@@ -1170,7 +1207,7 @@ class Ligand(Mol):
                                           fgroup='carboxylate',
                                           restype=self.hetid,
                                           resnr=self.position, reschain=self.chain,
-                                          location='ligand'))
+                                          location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
             if a.atomicnum == 15:  # It's a phosphor atom
                 if n_atoms_atomicnum.count(8) >= 3:  # It's a phosphoryl
                     for neighbor in [n for n in n_atoms if n.GetAtomicNum() == 8]:
@@ -1179,28 +1216,28 @@ class Ligand(Mol):
                                           fgroup='phosphoryl',
                                           restype=self.hetid,
                                           resnr=self.position, reschain=self.chain,
-                                          location='ligand'))
+                                          location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
                 if n_atoms_atomicnum.count(8) == 2:  # It's another phosphor-containing group #@todo (correct name?)
                     for neighbor in [n for n in n_atoms if n.GetAtomicNum() == 8]:
                         neighbor_orig_idx = self.Mapper.mapid(neighbor.GetIdx(), mtype='ligand', bsid=self.bsid)
                         a_set.append(data(atom=pybel.Atom(neighbor), atom_orig_idx=neighbor_orig_idx, type='O',
                                           fgroup='phosphor.other', restype=self.hetid,
                                           resnr=self.position,
-                                          reschain=self.chain, location='ligand'))
+                                          reschain=self.chain, location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
             if a.atomicnum == 7:  # It's a nitrogen atom
                 if n_atoms_atomicnum.count(6) == 2:  # It's imidazole/pyrrole or similar
                     a_set.append(data(atom=a, atom_orig_idx=a_orig_idx, type='N', fgroup='imidazole/pyrrole',
                                       restype=self.hetid, resnr=self.position, reschain=self.chain,
-                                      location='ligand'))
+                                      location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
             if a.atomicnum == 16:  # It's a sulfur atom
                 if True in [n.IsAromatic() for n in n_atoms] and not a.OBAtom.IsAromatic():  # Thiolate
                     a_set.append(data(atom=a, atom_orig_idx=a_orig_idx, type='S', fgroup='thiolate',
                                       restype=self.hetid, resnr=self.position, reschain=self.chain,
-                                      location='ligand'))
+                                      location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
                 if set(n_atoms_atomicnum) == {26}:  # Sulfur in Iron sulfur cluster
                     a_set.append(data(atom=a, atom_orig_idx=a_orig_idx, type='S', fgroup='iron-sulfur.cluster',
                                       restype=self.hetid, resnr=self.position, reschain=self.chain,
-                                      location='ligand'))
+                                      location='ligand', orig_atom=self.Mapper.id_to_atom(a_orig_idx)))
 
         return a_set
 
@@ -1247,6 +1284,7 @@ class PDBComplex:
         pdbparser = PDBParser(pdbpath, as_string=as_string)  # Parse PDB file to find errors and get additonal data
         # #@todo Refactor and rename here
         self.Mapper.proteinmap = pdbparser.proteinmap
+        self.Mapper.reversed_proteinmap = inv_map = {v: k for k, v in self.Mapper.proteinmap.iteritems()}
         self.modres = pdbparser.modres
         self.covalent = pdbparser.covalent
         self.altconf = pdbparser.altconformations
@@ -1270,8 +1308,12 @@ class PDBComplex:
         else:
             self.sourcefiles['filename'] = os.path.basename(self.sourcefiles['pdbcomplex'])
             self.protcomplex, self.filetype = read_pdb(self.sourcefiles['pdbcomplex'], as_string=as_string)
+
+        # Update the model in the Mapper class instance
+        self.Mapper.original_structure = self.protcomplex.OBMol
         write_message('PDB structure successfully read.\n')
 
+
         # Determine (temporary) PyMOL Name from Filename
         self.pymol_name = pdbpath.split('/')[-1].split('.')[0] + '-Protein'
         # Replace characters causing problems in PyMOL
@@ -1283,19 +1325,24 @@ class PDBComplex:
                 self.pymol_name = potential_name
         write_message("Pymol Name set as: '%s'\n" % self.pymol_name, mtype='debug')
 
-        self.protcomplex.OBMol.AddPolarHydrogens()
-        for atm in self.protcomplex:
-            self.atoms[atm.idx] = atm
-
         # Extract and prepare ligands
         ligandfinder = LigandFinder(self.protcomplex, self.altconf, self.modres, self.covalent, self.Mapper)
         self.ligands = ligandfinder.ligands
         self.excluded = ligandfinder.excluded
 
+        # Add polar hydrogens
+        self.protcomplex.OBMol.AddPolarHydrogens()
+        for atm in self.protcomplex:
+            self.atoms[atm.idx] = atm
+        write_message("Assigned polar hydrogens\n", mtype='debug')
+
         if len(self.excluded) != 0:
             write_message("Excluded molecules as ligands: %s\n" % ','.join([lig for lig in self.excluded]))
 
-        self.resis = [obres for obres in pybel.ob.OBResidueIter(self.protcomplex.OBMol) if obres.GetResidueProperty(0)]
+        if config.DNARECEPTOR:
+            self.resis = [obres for obres in pybel.ob.OBResidueIter(self.protcomplex.OBMol) if obres.GetName() in config.DNA+config.RNA]
+        else:
+            self.resis = [obres for obres in pybel.ob.OBResidueIter(self.protcomplex.OBMol) if obres.GetResidueProperty(0)]
 
         num_ligs = len(self.ligands)
         if num_ligs == 1:
