@@ -1,26 +1,14 @@
 """
 Protein-Ligand Interaction Profiler - Analyze and visualize protein-ligand interactions in PDB files.
 supplemental.py - Supplemental functions for PLIP analysis.
-Copyright 2014-2015 Sebastian Salentin, Joachim Haupt
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
 """
 
 # Compatibility
 from __future__ import print_function
+from __future__ import absolute_import
 
 # PLIP Modules
-import config
+import plip.modules.config as config
 
 # Python standard library
 import re
@@ -300,9 +288,9 @@ def ring_is_planar(ring, r_atoms):
 def classify_by_name(names):
     """Classify a (composite) ligand by the HETID(s)"""
     if len(names) > 3:  # Polymer
-        if len({'U', 'A', 'C', 'G'}.intersection(set(names))) != 0:
+        if len(set(config.RNA).intersection(set(names))) != 0:
             ligtype = 'RNA'
-        elif len({'DT', 'DA', 'DC', 'DG'}.intersection(set(names))) != 0:
+        elif len(set(config.DNA).intersection(set(names))) != 0:
             ligtype = 'DNA'
         else:
             ligtype = "POLYMER"
@@ -420,25 +408,30 @@ def read(fil):
 def readmol(path, as_string=False):
     """Reads the given molecule file and returns the corresponding Pybel molecule as well as the input file type.
     In contrast to the standard Pybel implementation, the file is closed properly."""
-    supported_formats = ['pdb', 'pdbqt', 'mmcif']
+    supported_formats = ['pdb', 'mmcif']
     obc = pybel.ob.OBConversion()
 
-    if as_string:
-        filestr = path
-    else:
-        with read(path) as f:
-            filestr = str(f.read())
     for sformat in supported_formats:
         obc.SetInFormat(sformat)
+        write_message("Detected {} as format. Now trying to read file with OpenBabel...\n".format(sformat), mtype='debug')
         mol = pybel.ob.OBMol()
-        obc.ReadString(mol, filestr)
-        if not mol.Empty():
-            if sformat == 'pdbqt':
-                write_message('[EXPERIMENTAL] Input is PDBQT file. Some features (especially visualization) might not '
-                        'work as expected. Please consider using PDB format instead.\n')
-            if sformat == 'mmcif':
-                write_message('[EXPERIMENTAL] Input is mmCIF file. Most features do currently not work with this format.\n')
-            return pybel.Molecule(mol), sformat
+
+        # Read molecules with single bond information
+        if as_string:
+            read_file = pybel.readstring(format=sformat, filename=path, opt={"s": None})
+        else:
+            read_file = pybel.readfile(format=sformat, filename=path, opt={"s": None})
+        try:
+            mymol = read_file.next()
+        except StopIteration:
+            sysexit(4, 'File contains no valid molecules.\n')
+
+        write_message("Molecule successfully read.\n", mtype='debug')
+
+        # Assign multiple bonds
+        mymol.OBMol.PerceiveBondOrders()
+        return mymol, sformat
+
     sysexit(4, 'No valid file format provided.')
 
 
