@@ -1,27 +1,16 @@
 """
 Protein-Ligand Interaction Profiler - Analyze and visualize protein-ligand interactions in PDB files.
 visualize.py - Visualization of PLIP results using PyMOL.
-Copyright 2014-2015 Sebastian Salentin
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
 """
 
+# Python standard library
+from __future__ import absolute_import
 
 # Own modules
-from supplemental import initialize_pymol, start_pymol, write_message, colorlog, sysexit
-import config
-from pymolplip import PyMOLVisualizer
-from plipremote import VisualizerData
+from plip.modules.supplemental import initialize_pymol, start_pymol, write_message, colorlog, sysexit
+import plip.modules.config as config
+from plip.modules.pymolplip import PyMOLVisualizer
+from plip.modules.plipremote import VisualizerData
 
 # Python Standard Library
 import json
@@ -114,6 +103,7 @@ def visualize_in_pymol(plcomplex):
 
 
     vis.make_initial_selections()
+
     vis.show_hydrophobic()  # Hydrophobic Contacts
     vis.show_hbonds()  # Hydrogen Bonds
     vis.show_halogen()  # Halogen Bonds
@@ -125,12 +115,18 @@ def visualize_in_pymol(plcomplex):
 
     vis.refinements()
 
-
     vis.zoom_to_ligand()
 
     vis.selections_cleanup()
+
     vis.selections_group()
     vis.additional_cleanup()
+    if config.DNARECEPTOR:
+        # Rename Cartoon selection to Line selection and change repr.
+        cmd.set_name('%sCartoon' % plcomplex.pdbid, '%sLines' % plcomplex.pdbid)
+        cmd.hide('cartoon', '%sLines' % plcomplex.pdbid)
+        cmd.show('lines', '%sLines' % plcomplex.pdbid)
+
     if config.PEPTIDES != []:
         filename = "%s_PeptideChain%s" % (pdbid.upper(), plcomplex.chain)
         if config.PYMOL:
